using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Configuration;
using System.Web.Configuration;
using System.Web.Security;
using System.Security.Permissions;
using System.Security.Cryptography;

public partial class enc_test2 : System.Web.UI.Page {

	const string isNotFoundText = "<font color=gray>Not Found</font>";
	const string isProtectedText = "<font color=green>Protected</font>";
	const string isUnprotectedText = "<font color=red>Unprotected</font>";
	const string protectText = "Protect this section using the Provider selected above";
	const string unprotectText = "Unprotect this section";

	protected void Page_Load(object sender, EventArgs e) {

		// Load Configuration and Section information.
		Configuration config = WebConfigurationManager.OpenWebConfiguration(Request.ApplicationPath);
		AppSettingsSection appSettings = config.Sections["appSettings"] as AppSettingsSection;
		ConnectionStringsSection connectionStrings = config.Sections["connectionStrings"] as ConnectionStringsSection;
		ConfigurationSection identity = config.Sections["identity"];
		ConfigurationSection sessionState = config.Sections["sessionState"];

		// appSettings
		if (appSettings != null && appSettings.Settings.Count > 0) {
			litAppSettingsEncrypted.Text = (appSettings.SectionInformation.IsProtected) ? isProtectedText : isUnprotectedText;
			btnAppSettings.Text = (appSettings.SectionInformation.IsProtected) ? unprotectText : protectText;
		} else {
			litAppSettingsEncrypted.Text = isNotFoundText;
			btnAppSettings.Visible = false;
		}

		// connectionStrings
		if (connectionStrings != null && connectionStrings.ConnectionStrings.Count > 0) {
			litConnectionStringsEncrypted.Text = (connectionStrings.SectionInformation.IsProtected) ? isProtectedText : isUnprotectedText;
			btnConnectionStrings.Text = (connectionStrings.SectionInformation.IsProtected) ? unprotectText : protectText;
		} else {
			litConnectionStringsEncrypted.Text = isNotFoundText;
			btnConnectionStrings.Visible = false;
		}

		//identity
		if (identity != null) {
			litIdentityEncrypted.Text = (identity.SectionInformation.IsProtected) ? isProtectedText : isUnprotectedText;
			btnIdentity.Text = (identity.SectionInformation.IsProtected) ? unprotectText : protectText;
		} else {
			litIdentityEncrypted.Text = isNotFoundText;
			btnIdentity.Visible = false;
		}

		// sessionState
		if (sessionState != null) {
			litSessionStateEncrypted.Text = (sessionState.SectionInformation.IsProtected) ? isProtectedText : isUnprotectedText;
			btnSessionState.Text = (sessionState.SectionInformation.IsProtected) ? unprotectText : protectText;
		} else {
			litSessionStateEncrypted.Text = isNotFoundText;
			btnSessionState.Visible = false;
		}

	}

	#region - Button Commands (btnProtect_Command) -

	protected void btnProtect_Command(object sender, CommandEventArgs e) {
		LinkButton button = sender as LinkButton;

		if (button.Text == protectText) {
			if (ddlProviders.SelectedIndex == 0) { litMessage.Text = "<font color=red>You must select a Provider.</font>"; return; }
			EncryptSection(ddlProviders.SelectedValue, e.CommandArgument.ToString());
		} else {
			DecryptSection(e.CommandArgument.ToString());
		}

		if (litMessage.Text == "") Response.Redirect(Request.Url.ToString());
	}

	#endregion

	#region - DropDownList Load Event (ddlProviders_Load) -
	/// <summary>Populates the ddlProviders DropDownList with Provider Names.</summary>
	protected void ddlProviders_Load(object sender, EventArgs e) {
		if (Page.IsPostBack) return;

		// Default ViewStates
		phDpapiProtectedConfigurationProvider.Visible = true;
		mvRsaCryptoServiceProvider.ActiveViewIndex = 0;

		// Load Section 'configProtectedData'
		Configuration config = WebConfigurationManager.OpenWebConfiguration(Request.ApplicationPath);
		ProtectedConfigurationSection configProtectedData = config.Sections["configProtectedData"] as ProtectedConfigurationSection;

		//System.Security.Cryptography.RSACryptoServiceProvider.CspKeyContainerInfo

		// Populate DropDownList
		for (int i = 0; i < configProtectedData.Providers.Count; i++) {
			ddlProviders.Items.Add(configProtectedData.Providers[i].Name);
			if (String.Compare(configProtectedData.Providers[i].Name, "MyDpapiProtectedConfigurationProvider", true) == 0) {
				phDpapiProtectedConfigurationProvider.Visible = false;
			} else if (String.Compare(configProtectedData.Providers[i].Name, "MyRsaCryptoServiceProvider", true) == 0) {
				mvRsaCryptoServiceProvider.ActiveViewIndex = 1;
			}
		}
	}
	#endregion

	#region - Private Methods (GetIdentity) -

	private string GetIdentity() {
		string value = System.Security.Principal.WindowsIdentity.GetCurrent().Name;

		if (value.IndexOf(@"\") > -1) value = value.Substring(value.LastIndexOf(@"\") + 1);

		return value;
	}

	#endregion

	#region - Encryption Methods (EncryptSection, EncryptSection, DecryptSection, TestProvider) -

	/// <summary>Encrypts specified section using protectionProvider.</summary>
	/// <param name="protectionProvider">Name of ProtectionProvider to use.  Examples: DataProtectionConfigurationProvider, RSAProtectedConfigurationProvider.</param>
	/// <param name="sectionName">Name of Section to Encrypt.  Example: connectionStrings.</param>
	/// <param name="newFileName">New FileName to save to.  If null, default location will be used.</param>
	public void EncryptSection(string protectionProvider, string sectionName, string newFileName) {

		// Load configuration and section
		Configuration config = WebConfigurationManager.OpenWebConfiguration(Request.ApplicationPath);
		ConfigurationSection section = config.Sections[sectionName];

		// Encrypt / Decrypt Section
		if (protectionProvider != null) section.SectionInformation.ProtectSection(protectionProvider);
		else section.SectionInformation.UnprotectSection();

		// Save
		try {
			if (newFileName != null) config.SaveAs(Server.MapPath(newFileName));
			else config.Save();
		} catch (Exception ex) {
			litMessage.Text = "An error occurred while attempting to save the config. ";//
			if (ex.ToString().IndexOf("read-only") > -1) {
				litMessage.Text += config.FilePath + " is readonly.";
			} else if (ex.ToString().IndexOf(" is denied.") > -1) {
				litMessage.Text += "Access to '" + config.FilePath + "' is denied.";
			} else if (ex.ToString().IndexOf("0x80070002") > -1) {
				litMessage.Text += @"Path \Documents and Settings\" + GetIdentity() + @"\Application Data\Microsoft\Crypto\RSA\ does not exist.<br><br>" + ex.ToString().Replace("\n", "<br>");
			} else {
				litMessage.Text += "<br><br>" + ex.ToString().Replace("\n", "<br>");
			}
			litMessage.Text += "<br><br>";
		}
	}

	/// <summary>Encrypts specified section using protectionProvider.</summary>
	/// <param name="protectionProvider">Name of ProtectionProvider to use.  Examples: DataProtectionConfigurationProvider, RSAProtectedConfigurationProvider.</param>
	/// <param name="sectionName">Name of Section to Encrypt.  Example: connectionStrings.</param>
	public void EncryptSection(string protectionProvider, string sectionName) {
		EncryptSection(protectionProvider, sectionName, null);
	}

	/// <summary>Decrypts specified section.</summary>
	/// <param name="sectionName"></param>
	/// <param name="newFileName"></param>
	public void DecryptSection(string sectionName, string newFileName) {
		EncryptSection(null, sectionName, newFileName);
	}

	/// <summary>Decrypts specified section.</summary>
	/// <param name="sectionName"></param>
	public void DecryptSection(string sectionName) {
		EncryptSection(null, sectionName, null);
	}

	/// <summary></summary>
	/// <param name="protectionProvider"></param>
	/// <returns></returns>
	public bool TestProvider(string protectionProvider) {
		bool value = true;

		try {
			Configuration config = WebConfigurationManager.OpenWebConfiguration(Request.ApplicationPath);
			ConnectionStringsSection connectionStrings = config.Sections["connectionStrings"] as ConnectionStringsSection;
			connectionStrings.SectionInformation.ProtectSection(protectionProvider);
		} catch (Exception ex) {
			throw new Exception(String.Format("TestProvider(\"{0}\") failed.", protectionProvider), ex);
			value = false;
		}

		return value;
	}

	#endregion

}
